// Copyright JGX Software Solutions, 2018.

// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
// THIS SOFTWARE.

#include <vector>
#include <fstream>
#include "Entities.h"
#include "V3.h"

using namespace std;

vector<node *> vNodes;
vector<element *> vElements;

int numNodes = 0;
int numElements = 0;


node * newNode(void) {
	numNodes++;
	auto pNode = new node(numNodes);
	vNodes.push_back(pNode);
	return(pNode);
}

element * newElement(void) {
	numElements++;
	auto pElement = new element(numElements);
	vElements.push_back(pElement);
	return(pElement);
}

void clearMesh(void) {
	terminateEntities();
	vNodes.clear();
	vElements.clear();
	numNodes = 0;
	numElements = 0;
}

void terminateEntities(void) {
	for (auto x : vNodes) { if (x) delete x; }
	for (auto x : vElements) { if (x) delete x; }
}

void exportSurfaceMesh(ofstream & fs) {

	vector<node *> & pnodes = vNodes;
	vector<element *> & pelems = vElements;

	fs << "nodes" << endl << "/" << endl;

	for (int i = 0; i < pnodes.size(); ++i) {

		fs << pnodes[i]->getCoords()[0] << " "
		   << pnodes[i]->getCoords()[1] << " "
		   << pnodes[i]->getCoords()[2] << " > " << pnodes[i]->getName() << endl;
	}

	fs << "/" << endl << endl;
	fs << "elements" << endl << "/" << endl;

	for (int i = 0; i < pelems.size(); ++i) {
		for (int j = 0; j < (pelems[i]->getNodes()).size(); ++j) {
			fs << (pelems[i]->getNodes())[j]->getName() << " ";
		}
		fs << endl;
	}

	fs << "/" << endl << endl;

	fs << "elements" << endl << "/" << endl;

	for (int i = 0; i < pelems.size(); ++i) {
		for (int j = 0; j < (pelems[i]->getNodes()).size(); ++j) {
			auto pnext = (pelems[i]->getNodes())[0];
			if (j < (pelems[i]->getNodes()).size() - 1) pnext = (pelems[i]->getNodes())[j + 1];
			fs << (pelems[i]->getNodes())[j]->getName() << " " << pnext->getName() << endl;
		}
	}

	fs << "/" << endl << endl;

}



