// Copyright JGX Software Solutions, 2018.

// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
// THIS SOFTWARE.

#pragma once

#include <string>
#include <vector>
#include "V3.h"

using namespace std;


class entity {
private:
	string name;
public:
	entity() {}
	entity(const string & Name) : name(Name) {}
	void putName(const string &Name) { name = Name; }
	string getName() { return name; }
};


class node : public entity {
private:
	V3 coords;

	int index = 0;

	// The following are used by and should only be used by the meshing functionality.
	int nConn = 0;         // Elemental connectivity of the node.
	bool bFixed = false;    // Do not optimize this node if 'true'.
	bool bAdjusted = false; // Do not adjust this node during the smoothing process if 'true'.

	V3 smoothVector = 0.0;
public:
	node() : entity() {}
	node(const string & Name) : entity(Name) {}
	node(const string & Name, V3 Coords) : entity(Name), coords(Coords) {}
	node(int iname) { entity::putName(to_string(iname)); coords = V3{ 0.0 }; }
	node(int iname, const V3 & Coords) { entity::putName(to_string(iname)); coords = Coords;}
	~node() {}

	void putName(const string &Name) { entity::putName(Name); }
	string getName() { return entity::getName(); }
	V3 getCoords() { return coords; }
	void putCoords(const V3& nCoords) { coords = nCoords; }

	int getConnectivity() { return nConn; }
	void putConnectivity(int nc) { nConn = nc; }
	void incrementConnectivity() { nConn++; }
	void decrementConnectivity() { nConn--; }

	bool isFixed() { return bFixed; }
	void putFixed(bool bf) { bFixed = bf; }
	void setFixed() { bFixed = true; }

	bool isAdjusted() { return bAdjusted; }
	void putAdjusted(bool bf) { bAdjusted = bf; }
	void setAdjusted() { bAdjusted = true; }

	void putSmoothVector(V3 sv) { smoothVector = sv; }
	void incrementSmoothVector(V3 sv) { smoothVector += sv; }
	V3 getSmoothVector() { return smoothVector; }

};


class element : public entity {
private:
	vector<node *> nodeList;

	double elsize = 0.0;
public:
	element() : entity() {}
	element(const string & Name) : entity(Name) {}
	element(const string & Name, const vector<node *> & nlist) : entity(Name), nodeList(nlist) {}
	element(int iname) { entity::putName(to_string(iname)); }
	element(int iname, const vector<node *> & nlist) { entity::putName(to_string(iname)); nodeList = nlist; }
	~element() {}

	void putName(const string &Name) { entity::putName(Name); }
	string getName() { return entity::getName(); }

	void putNodes(const vector<node *> &  nlist) { nodeList = nlist; }
	vector<node *> & getNodes(void) { return nodeList; }
};


node * newNode(void);
element * newElement(void);
void terminateEntities(void);
void clearMesh(void);
