// Copyright JGX Software Solutions, 2018.

// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
// THIS SOFTWARE.

#include <cassert>
#include <cmath>
#include <fstream>
#include "V3.h"

using namespace std;


V3::V3(const V3 & vr) {
	// Copy constructor.
	Data[0] = vr.Data[0];
	Data[1] = vr.Data[1];
	Data[2] = vr.Data[2];
}

V3& V3::operator = (const V3 &vr) {
	Data[0] = vr.Data[0];
	Data[1] = vr.Data[1];
	Data[2] = vr.Data[2];
	return *this;
}

V3& V3::operator = (double d[3]) {
	Data[0] = d[0];
	Data[1] = d[1];
	Data[2] = d[2];
	return *this;
}

V3& V3::operator = (double d) {
	Data[0] = d;
	Data[1] = d;
	Data[2] = d;
	return *this;
}

V3& V3::operator += (const V3 &vr) {
	Data[0] = Data[0] + vr.Data[0];
	Data[1] = Data[1] + vr.Data[1];
	Data[2] = Data[2] + vr.Data[2];
	return *this;
}

V3& V3::operator -= (const V3 &vr) {
	Data[0] = Data[0] - vr.Data[0];
	Data[1] = Data[1] - vr.Data[1];
	Data[2] = Data[2] - vr.Data[2];
	return *this;
}

V3& V3::operator *= (double fac) {
	Data[0] = Data[0] * fac;
	Data[1] = Data[1] * fac;
	Data[2] = Data[2] * fac;
	return *this;
}

V3& V3::operator /= (double fac) {
	Data[0] = Data[0] / fac;
	Data[1] = Data[1] / fac;
	Data[2] = Data[2] / fac;
	return *this;
}

V3 operator + (const V3 & a, const V3 & b) {
	double c[3];

	c[0] = a[0] + b[0];
	c[1] = a[1] + b[1];
	c[2] = a[2] + b[2];

	return(V3(c));
}

V3 operator - (const V3 & a, const V3 & b) {
	double c[3];

	c[0] = a[0] - b[0];
	c[1] = a[1] - b[1];
	c[2] = a[2] - b[2];

	return(V3(c));
}

V3 operator * (const V3 & a, double d) {
	V3 c;
	c[0] = a[0] * d;
	c[1] = a[1] * d;
	c[2] = a[2] * d;
	return(c);
}

V3 operator * (double d, const V3 & a) {
	V3 c;
	c[0] = a[0] * d;
	c[1] = a[1] * d;
	c[2] = a[2] * d;
	return(c);
}

double operator * (const V3 & a, const V3 & b) {
	// Dot product.
	double c = 0.0;
	c += a[0] * b[0];
	c += a[1] * b[1];
	c += a[2] * b[2];
	return(c);
}

V3 operator / (const V3 & a, double d) {
	V3 c;
	c[0] = a[0] / d;
	c[1] = a[1] / d;
	c[2] = a[2] / d;
	return(c);
}

double& V3::operator[] (const int index) // for non-const objects: can be used for assignment
{
	assert((index >= 0 && index < 3) && "V3 [] out of bounds");
	return Data[index];
}

const double& V3::operator[] (const int index) const // for const objects: can only be used for access
{
	assert((index >= 0 && index < 3) && "V3 [] out of bounds");
	return Data[index];
}

V3 operator ^ (const V3 & a, const V3 & b) {
	// Cross product.
	double c[3];
	c[0] = a[1] * b[2] - a[2] * b[1];
	c[1] = a[2] * b[0] - a[0] * b[2];
	c[2] = a[0] * b[1] - a[1] * b[0];
	return(V3(c));
}

double V3::length() const {
	return(sqrt(pow(Data[0], 2) + pow(Data[1], 2) + pow(Data[2], 2)));
}

