// Copyright JGX Software Solutions, 2018.

// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
// THIS SOFTWARE.

#pragma once

#include <fstream>

using namespace std;

class V3 {
private:
	double Data[3];
public:
	V3() : Data{ 0.0, 0.0, 0.0 } { }                                  // Overrided default constructor	
																	  
	// Some parametrized constructors.

	V3(double d) : Data{ d, d, d } { }                                // e.g. V3 va {0.0};
	V3(double d1, double d2, double d3) : Data{ d1, d2, d3 } { }      // e.g. V3 va {5.0, 5.0, 4.0 };
	V3(const double d[3]) : Data{ d[0], d[1], d[2] } { }              // e.g. double r[3]; V3 va {r};// e.g. V3 va;

	V3(const V3 &); // Copy constructor.                              // e.g. V3 e = d;

	V3& operator = (const V3 &);                                      // e.g. V3 e; e = d;
	V3& operator = (double d[3]);                                     // e.g. V3 e; double r[3]; e=r;
	V3& operator = (double d);                                        // e.g. V3 e; e = 4.0;

	V3& operator += (const V3 &);
	V3& operator -= (const V3 &);
	V3& operator *= (double);
	V3& operator /= (double);

	double& operator[] (const int index);
	const double& operator[] (const int index) const;

	double length(void) const;
};

V3 operator + (const V3 &, const V3 &);
V3 operator - (const V3 &, const V3 &);

V3 operator ^ (const V3 &, const V3 &);

V3 operator * (const V3 &, double);
V3 operator * (double, const V3 &);
double operator * (const V3 &, const V3 &);                // scalar product
V3 operator / (const V3 &, double);


